/**
 * @file g_actor.c
 */

/*
Copyright (C) 2002-2009 UFO: Alien Invasion.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "g_local.h"

/**
 * @brief Checks whether the given edict is a living actor
 * @param[in] ent The edict to perform the check for
 * @sa LE_IsLivingActor
 */
qboolean G_IsLivingActor (const edict_t *ent)
{
	return G_IsActor(ent) && !G_IsDead(ent);
}

/**
 * @brief Turns an actor around
 * @param[in] ent the actor (edict) we are talking about
 * @param[in] dir the direction to turn the edict into, might be an action
 * @return Bitmask of visible (VIS_*) values
 * @sa G_CheckVisTeam
 */
int G_ActorDoTurn (edict_t * ent, byte dir)
{
	float angleDiv;
	const byte *rot;
	int i, num;
	int status;

	assert(ent->dir < CORE_DIRECTIONS);
	assert(dir < PATHFINDING_DIRECTIONS);

	/*
	 * If dir is at least CORE_DIRECTIONS but less than FLYING_DIRECTIONS,
	 * then the direction is an action.
	 */
	/** @todo If performing an action, ensure the actor is facing the direction
	 *  needed to perform the action if needed (climbing ladders).
	 */
	if (dir >= CORE_DIRECTIONS && dir < FLYING_DIRECTIONS)
		return 0;

	/* Clamp dir between 0 and CORE_DIRECTIONS - 1. */
	dir &= (CORE_DIRECTIONS - 1);
	assert(dir < CORE_DIRECTIONS);

	/* Return if no rotation needs to be done. */
	if (ent->dir == dir)
		return 0;

	/* calculate angle difference */
	angleDiv = directionAngles[dir] - directionAngles[ent->dir];
	if (angleDiv > 180.0)
		angleDiv -= 360.0;
	if (angleDiv < -180.0)
		angleDiv += 360.0;

	/* prepare rotation */
	if (angleDiv > 0) {
		rot = dvleft;
		num = (angleDiv + 22.5) / 45.0;
	} else {
		rot = dvright;
		num = (-angleDiv + 22.5) / 45.0;
	}

	/* do rotation and vis checks */
	status = 0;

	/* check every angle in the rotation whether something becomes visible */
	for (i = 0; i < num; i++) {
		ent->dir = rot[ent->dir];
		assert(ent->dir < CORE_DIRECTIONS);
		status |= G_CheckVisTeam(ent->team, NULL, qfalse, ent);
	}

	if (status & VIS_STOP) {
		/* send the turn */
		G_ActorTurnEvent(ent);
	}

	return status;
}

/**
 * @brief Send the turn event for the given entity
 * @param ent The entity to send the turn event for
 * @note Every player that can see this ent will reveive the turn event data
 * @note Make sure that the direction to turn into is already set
 */
void G_ActorTurnEvent (const edict_t* ent)
{
	gi.AddEvent(G_VisToPM(ent->visflags), EV_ACTOR_TURN);
	gi.WriteShort(ent->number);
	gi.WriteByte(ent->dir);
}

/**
 * @brief Report and handle death of an actor
 */
void G_ActorDie (edict_t * ent, int state, edict_t *attacker)
{
	assert(ent);

	Com_DPrintf(DEBUG_GAME, "G_ActorDie: kill actor on team %i\n", ent->team);
	switch (state) {
	case STATE_DEAD:
		ent->state |= (1 + rand() % MAX_DEATH);
		break;
	case STATE_STUN:
		/**< @todo Is there a reason this is reset? We _may_ need that in the future somehow.
		 * @sa CL_ActorDie */
		ent->STUN = 0;
		ent->state = state;
		break;
	default:
		Com_DPrintf(DEBUG_GAME, "G_ActorDie: unknown state %i\n", state);
		break;
	}
	VectorSet(ent->maxs, PLAYER_WIDTH, PLAYER_WIDTH, PLAYER_DEAD);
	gi.LinkEdict(ent);
	level.num_alive[ent->team]--;
	/* send death */
	gi.AddEvent(G_VisToPM(ent->visflags), EV_ACTOR_DIE);
	gi.WriteShort(ent->number);
	gi.WriteShort(attacker ? attacker->number : SKIP_LOCAL_ENTITY);
	gi.WriteShort(ent->state);

	/* handle inventory - drop everything to floor edict (but not the armour) */
	G_InventoryToFloor(ent);

	/* check if the player appears/perishes, seen from other teams */
	G_CheckVis(ent, qtrue);

	/* check if the attacker appears/perishes, seen from other teams */
	if (attacker)
		G_CheckVis(attacker, qtrue);

	/* calc new vis for this player */
	G_CheckVisTeam(ent->team, NULL, qfalse, attacker);
}

/**
 * @brief Add a new wound to the actor in a random location.
 * @param[in] target Who is injured
 * @param[in] damage How much damage is being inflicted
 */
void G_ActorDamage (edict_t *target, const int damage)
{
	wound_t *newWound;
	int randomValue;

	/**< Sanity check, don't add a wound if no damage was done. */
	if(damage <= 0)
		return;

	newWound = malloc(sizeof(wound_t));
	/**< First determine where the wound is */
	randomValue = rand() % (WOUND_HEAD_PROBABILITY + WOUND_CHEST_PROBABILITY + WOUND_ARMS_PROBABILITY + WOUND_LEGS_PROBABILITY);
	if(randomValue < WOUND_HEAD_PROBABILITY) {
		newWound->location = WOUND_HEAD;
	}
	else if(randomValue < WOUND_HEAD_PROBABILITY + WOUND_CHEST_PROBABILITY) {
		newWound->location = WOUND_CHEST;
	}
	else if(randomValue < WOUND_HEAD_PROBABILITY + WOUND_CHEST_PROBABILITY + WOUND_ARMS_PROBABILITY) {
		newWound->location = WOUND_ARMS;
	}
	else if(randomValue < WOUND_HEAD_PROBABILITY + WOUND_CHEST_PROBABILITY + WOUND_ARMS_PROBABILITY + WOUND_LEGS_PROBABILITY) {
		newWound->location = WOUND_LEGS;
	}
	/**< This should *never* happen, put out an error or something */
	else {
		/**< WOUND_LEGS is probably the highest enum value, so % by it. */
		newWound->location %= WOUND_LEGS;
	}

	/**< Next determine if it is a wound */
	if(damage < WOUND_MINIMUM_DAMAGE) {
		newWound->affectSkill = 0;
		newWound->treated = 1;
	}
	else {
		newWound->affectSkill = 1;
		newWound->treated = 0;
	}

	/**< Finally set the damage of the wound */
	newWound->value = damage;

	/**< The wound is created, just add it to the target now */
	LIST_AddPointer(&target->chr.wounds, newWound);
	/**< Subtract the damage from the current hit points. */
	target->HP -= damage;
}

/**
 * @brief Return the decrease in accuracy due to wounds for an actor.
 * This increase is a percentage based on hit points currently.
 * @TODO Maybe make this calculation scripted in the future.
 * @param[in] target The injured actor we are calculating the decrease in accuracy for.
 */
int G_ActorAccuracyDecreaseByWounds (edict_t *target)
{
	linkedList_t *woundList;
	wound_t *wound;
	int armWounds = 0, headWounds = 0, returnValue = 0;

	/**< Calculate all arm wounds, we'll have to go through the wound linkedlist to count all arm wounds. */
	woundList = target->chr.wounds;
	while (woundList) {
		wound = (wound_t *)woundList->data;
		if ((wound->location == WOUND_ARMS) && wound->affectSkill) {
			if (wound->treated) {
				armWounds += wound->value / 2;
			}
			else {
				armWounds += wound->value;
			}
		}
		woundList = woundList->next;
	}
	/**< @todo: Value of all arm wounds is now stored in armWounds, do something with it here. */
	returnValue += armWounds / target->chr.HP;

	/**< Calculate all head wounds, we'll have to go through the wound linkedlist to count all head wounds. */
	woundList = target->chr.wounds;
	while (woundList) {
		wound = (wound_t *)woundList->data;
		if ((wound->location == WOUND_HEAD) && wound->affectSkill) {
			if (wound->treated) {
				headWounds += wound->value / 2;
			}
			else {
				headWounds += wound->value;
			}
		}
		woundList = woundList->next;
	}
	/**< @todo: Value of all head wounds is now stored in headWounds, do something with it here. */
	returnValue += headWounds / target->chr.HP;

	return returnValue;
}

/**
 * @brief Return the increase in turn units used by an actor due to wounds.
 * This increase is a percentage based on hit points currently.
 * @TODO Maybe make this calculation scripted in the future.
 * @param[in] target The injured actor we are calculating the increase in turn units for.
 */
int G_ActorShootTUIncreaseByWounds (edict_t *target)
{
	linkedList_t *woundList;
	wound_t *wound;
	int armWounds = 0, returnValue = 0;

	/**< Calculate all arm wounds, we'll have to go through the wound linkedlist to count all arm wounds. */
	woundList = target->chr.wounds;
	while (woundList) {
		wound = (wound_t *)woundList->data;
		if ((wound->location == WOUND_ARMS) && wound->affectSkill) {
			if (wound->treated) {
				armWounds += wound->value / 2;
			}
			else {
				armWounds += wound->value;
			}
		}
		woundList = woundList->next;
	}
	/**< @todo: Value of all arm wounds is now stored in armWounds, do something with it here. */
	returnValue += armWounds / target->chr.HP;

	return returnValue;
}

/**
 * @brief Return the increase in turn units to move used by an actor due to wounds.
 * This increase is a percentage based on hit points currently.
 * @TODO Maybe make this calculation scripted in the future.
 * @param[in] target The injured actor we are calculating the increase in turn units for.
 */
int G_ActorMoveTUIncreaseByWounds (character_t *target)
{
	linkedList_t *woundList;
	wound_t *wound;
	int legWounds = 0, returnValue = 0;

	/**< Calculate all leg wounds, we'll have to go through the wound linkedlist to count all leg wounds. */
	woundList = target->wounds;
	while (woundList) {
		wound = (wound_t *)woundList->data;
		if ((wound->location == WOUND_LEGS) && wound->affectSkill) {
			if (wound->treated) {
				legWounds += wound->value / 2;
			}
			else {
				legWounds += wound->value;
			}
		}
		woundList = woundList->next;
	}
	/**< @todo: Value of all leg wounds is now stored in legWounds, do something with it here. */
	returnValue += legWounds / target->HP;

	return returnValue;
}

/**
 * @brief Return the increase in turn units to reaction fire, used by an actor, due to wounds.
 * This increase is a percentage based on hit points currently.
 * @TODO Maybe make this calculation scripted in the future.
 * @param[in] target The injured actor we are calculating the increase in turn units for.
 */
int G_ActorReactionFireTUIncreaseByWounds (edict_t *target)
{
	linkedList_t *woundList;
	wound_t *wound;
	int chestWounds = 0, returnValue = 0;

	/**< Calculate all leg wounds, we'll have to go through the wound linkedlist to count all leg wounds. */
	woundList = target->chr.wounds;
	while (woundList) {
		wound = (wound_t *)woundList->data;
		if ((wound->location == WOUND_CHEST) && wound->affectSkill) {
			if (wound->treated) {
				chestWounds += wound->value / 2;
			}
			else {
				chestWounds += wound->value;
			}
		}
		woundList = woundList->next;
	}
	/**< @todo: Value of all leg wounds is now stored in chestWounds, do something with it here. */
	returnValue += chestWounds / target->chr.HP;

	return returnValue;
}

/**
 * @brief Treat the actor to slightly heal, and prevent bleeding of untreated combat wounds.
 * @note Type of healing is 1 for wounds, 2 for morale, 3 for stun.
 * @param[in] target Who's wounds are being treated.
 * @param[in] damage /1000=type of healing, %1000 = amount of healing.
 */
void G_ActorTreat (edict_t *target, const int damage)
{
	int type, value, index, amount, woundNumber;
	linkedList_t *woundList;
	wound_t *treatWound;

	/**< Determine the type and value of the healing. */
	type = damage / 1000;
	value = damage % 1000;

	/**< Sanity check on 'type' here. */
	if (type < 0)
		type = -type;

	/**< Sanity check on 'value' here. */
	if (value < 0)
		value = -value;

	if ( type == HEAL ) {
	/**< First we locate the worst wound on the person being treated */
		index = 0;
		amount = 0;
		woundNumber = 0;
		woundList = target->chr.wounds;
		while (woundList) {
			treatWound = (wound_t *)woundList->data;
			if ((!treatWound->treated) && (treatWound->value > amount)) {
			    amount = treatWound->value;
			    woundNumber = index;
			}
			index++;
			woundList = woundList->next;
		}
		woundList = target->chr.wounds;
		if (woundList) {
			treatWound = (wound_t *)LIST_GetByIdx(woundList, woundNumber);
			if (treatWound->treated == 0) {
				/**< Heal the wound a bit. The damage that was passed is in going to be used as
					 the divisor. This means the lower the damage, the more powerful the healing.
					 1 is the most powerful healing and it means the wound is completely healed.
					 999 is the weakest healing, and it means nothing happens (probably). */
				if (value) {
					treatWound->value -= treatWound->value / value;
					treatWound->treated = 1;
					target->HP += treatWound->value / value;
				}
				else
					gi.dprintf("G_ActorTreat: Script error for medikit HEAL type, value was 0.\n");

				if (!treatWound->value)
					LIST_Remove(&target->chr.wounds, woundList);
			}
		}
	}
	else if (type == MORALE) {
		target->morale += value;
		target->morale = min (target->morale, 100);
		target->morale = max (target->morale, 0);
	}
	else if (type == STUN) {
		target->STUN -= value;
		target->STUN = min (target->STUN, 255);
		target->STUN = max (target->STUN, 0);
	}
	else {
		gi.dprintf("G_ActorTreat: Undefined type (%i) with value %i.\n", type, value);
	}
}

/**
 * @brief Moves an item inside an inventory. Floors are handled special.
 * @param[in] entNum The edict number of the selected/used edict/soldier.
 * @param[in] from The container (-id) the item should be moved from.
 * @param[in] fItem The item you want to move.
 * @param[in] to The container (-id) the item should be moved to.
 * @param[in] tx x position where you want the item to go in the destination container
 * @param[in] ty y position where you want the item to go in the destination container
 * @param[in] checkaction Set this to qtrue if you want to check for TUs, otherwise qfalse.
 * @param[in] quiet Set this to qfalse to prevent message-flooding.
 * @sa event PA_INVMOVE
 * @sa AI_ActorThink
 */
void G_ActorInvMove (int entNum, const invDef_t * from, invList_t *fItem, const invDef_t * to, int tx,
		int ty, qboolean checkaction, qboolean quiet)
{
	player_t *player;
	edict_t *ent, *floor;
	invList_t *ic;
	qboolean newFloor;
	item_t item;
	int mask;
	inventory_action_t ia;
	int msglevel;
	invList_t fItemBackup;
	const invList_t* fItemBackupPtr;
	int fx, fy;

	ent = g_edicts + entNum;
	player = G_PLAYER_FROM_ENT(ent);
	msglevel = quiet ? PRINT_NONE : PRINT_HUD;

	assert(fItem);
	assert(fItem->item.t);

	/* Store the location/item of 'from' BEFORE actually moving items with Com_MoveInInventory. */
	fItemBackup = *fItem;
	fItemBackupPtr = fItem;

	/* Get first used bit in item. */
	Com_GetFirstShapePosition(fItem, &fx, &fy);
	fx += fItem->x;
	fy += fItem->y;

	/* Check if action is possible */
	/* TUs are 1 here - but this is only a dummy - the real TU check is done in the inventory functions below */
	if (checkaction && !G_ActionCheck(player, ent, 1, quiet))
		return;

	/* "get floor ready" - searching for existing floor-edict*/
	floor = G_GetFloorItems(ent); /* Also sets FLOOR(ent) to correct value. */
	if (to->id == gi.csi->idFloor && !floor) {
		/* We are moving to the floor, but no existing edict for this floor-tile found -> create new one */
		floor = G_SpawnFloor(ent->pos);
		newFloor = qtrue;
	} else if (from->id == gi.csi->idFloor && !floor) {
		/* We are moving from the floor, but no existing edict for this floor-tile found -> thi should never be the case. */
		gi.dprintf("G_ClientInvMove: No source-floor found.\n");
		return;
	} else {
		/* There already exists an edict for this floor-tile. */
		newFloor = qfalse;
	}

	/* search for space */
	if (tx == NONE) {
		if (ty != NONE)
			gi.dprintf("G_ClientInvMove: Error: ty != NONE, it is %i.\n", ty);

		ic = Com_SearchInInventory(&ent->i, from, fItem->x, fItem->y);
		if (ic)
			Com_FindSpace(&ent->i, &ic->item, to, &tx, &ty, fItem);
	}
	if (tx == NONE) {
		if (ty != NONE)
			gi.error("G_ClientInvMove: Error: ty != NONE, it is %i.\n", ty);
		return;
	}

	/* Try to actually move the item and check the return value */
	ia = Com_MoveInInventory(&ent->i, from, fItem, to, tx, ty, checkaction ? &ent->TU : NULL, &ic);
	switch (ia) {
	case IA_NONE:
		/* No action possible - abort */
		return;
	case IA_NOTIME:
		G_PlayerPrintf(player, msglevel, _("Can't perform action - not enough TUs!\n"));
		return;
	case IA_NORELOAD:
		G_PlayerPrintf(player, msglevel,
				_("Can't perform action - weapon already fully loaded with the same ammunition!\n"));
		return;
	default:
		/* Continue below. */
		assert(ic);
		break;
	}

	assert(gi.csi->idFloor >= 0 && gi.csi->idFloor < MAX_CONTAINERS);

	/* successful inventory change; remove the item in clients */
	if (from->id == gi.csi->idFloor) {
		/* We removed an item from the floor - check how the client
		 * needs to be updated. */
		assert(!newFloor);
		if (FLOOR(ent)) {
			/* There is still something on the floor. */
			FLOOR(floor) = FLOOR(ent);
			/* Tell the client to remove the item from the container */
			gi.AddEvent(G_VisToPM(floor->visflags), EV_INV_DEL);
			gi.WriteShort(floor->number);
			gi.WriteByte(from->id);
			gi.WriteByte(fx);
			gi.WriteByte(fy);
		} else {
			/* Floor is empty, remove the edict (from server + client) if we are
			 * not moving to it. */
			if (to->id != gi.csi->idFloor) {
				gi.AddEvent(G_VisToPM(floor->visflags), EV_ENT_PERISH);
				gi.WriteShort(floor->number);
				G_FreeEdict(floor);
			}
		}
	} else {
		/* Tell the client to remove the item from the container */
		gi.AddEvent(G_TeamToPM(ent->team), EV_INV_DEL);
		gi.WriteShort(entNum);
		gi.WriteByte(from->id);
		gi.WriteByte(fx);
		gi.WriteByte(fy);
	}

	/* send tu's */
	G_SendStats(ent);

	item = ic->item;

	if (ia == IA_RELOAD || ia == IA_RELOAD_SWAP) {
		/* reload */
		if (to->id == gi.csi->idFloor) {
			assert(!newFloor);
			assert(FLOOR(floor) == FLOOR(ent));
			mask = G_VisToPM(floor->visflags);
		} else {
			mask = G_TeamToPM(ent->team);
		}

		/* send ammo message to all --- it's fun to hear that sound */
		gi.AddEvent(PM_ALL, EV_INV_RELOAD);
		gi.WriteShort(to->id == gi.csi->idFloor ? floor->number : entNum);
		gi.WriteByte(item.t->ammo);
		gi.WriteByte(item.m->idx);
		gi.WriteByte(to->id);
		gi.WriteByte(ic->x);
		gi.WriteByte(ic->y);

		if (ia == IA_RELOAD) {
			gi.EndEvents();
			return;
		} else { /* ia == IA_RELOAD_SWAP */
			if (!fItemBackupPtr) {
				gi.dprintf("G_ClientInvMove: Didn't find invList of the item you're moving\n");
				gi.EndEvents();
				return;
			}
			item = fItemBackup.item;
			to = from;
			tx = fItemBackup.x;
			ty = fItemBackup.y;
		}
	}

	/* add it */
	if (to->id == gi.csi->idFloor) {
		/* We moved an item to the floor - check how the client needs to be updated. */
		if (newFloor) {
			/* A new container was created for the floor. */
			assert(FLOOR(ent));
			/* we have to link the temp floor container to the new floor edict */
			FLOOR(floor) = FLOOR(ent);
			/* Send item info to the clients */
			G_CheckVis(floor, qtrue);
		} else {
			/* Add the item to an already existing floor edict - the floor container that
			 * is already linked might be from a different entity */
			FLOOR(floor) = FLOOR(ent);
			/* Tell the client to add the item to the container. */
			gi.AddEvent(G_VisToPM(floor->visflags), EV_INV_ADD);
			gi.WriteShort(floor->number);
			gi.WriteShort(INV_INVENTORY_BYTES);
			assert(item.t);
			G_WriteItem(item, to, tx, ty);
		}
	} else {
		/* Tell the client to add the item to the container. */
		gi.AddEvent(G_TeamToPM(ent->team), EV_INV_ADD);
		gi.WriteShort(entNum);
		gi.WriteShort(INV_INVENTORY_BYTES);
		assert(item.t);
		G_WriteItem(item, to, tx, ty);
	}

	/* Update reaction firemode when something is moved from/to a hand. */
	if (from->id == gi.csi->idRight || to->id == gi.csi->idRight) {
		gi.AddEvent(G_TeamToPM(ent->team), EV_INV_HANDS_CHANGED);
		gi.WriteShort(entNum);
		gi.WriteShort(ACTOR_HAND_RIGHT);
	} else if (from->id == gi.csi->idLeft || to->id == gi.csi->idLeft) {
		gi.AddEvent(G_TeamToPM(ent->team), EV_INV_HANDS_CHANGED);
		gi.WriteShort(entNum);
		gi.WriteShort(ACTOR_HAND_LEFT);
	}

	/* Other players receive weapon info only. */
	mask = G_VisToPM(ent->visflags) & ~G_TeamToPM(ent->team);
	if (mask) {
		if (from->id == gi.csi->idRight || from->id == gi.csi->idLeft) {
			gi.AddEvent(mask, EV_INV_DEL);
			gi.WriteShort(entNum);
			gi.WriteByte(from->id);
			gi.WriteByte(fx);
			gi.WriteByte(fy);
		}
		if (to->id == gi.csi->idRight || to->id == gi.csi->idLeft) {
			gi.AddEvent(mask, EV_INV_ADD);
			gi.WriteShort(entNum);
			gi.WriteShort(INV_INVENTORY_BYTES);
			assert(item.t);
			G_WriteItem(item, to, tx, ty);
		}
	}
	gi.EndEvents();
}

/**
 * @brief Reload weapon with actor.
 * @sa AI_ActorThink
 */
void G_ActorReload (int entnum, shoot_types_t st, qboolean quiet)
{
	edict_t *ent;
	invList_t *ic;
	invList_t *icFinal;
	invDef_t * hand;
	objDef_t *weapon;
	int tu;
	int containerID;
	invDef_t *bestContainer;

	ent = g_edicts + entnum;

	/* search for clips and select the one that is available easily */
	icFinal = NULL;
	tu = 100;
	hand = (st == ST_RIGHT_RELOAD) ? INVDEF(gi.csi->idRight) : INVDEF(gi.csi->idLeft);
	bestContainer = NULL;

	if (CONTAINER(ent, hand->id)) {
		weapon = CONTAINER(ent, hand->id)->item.t;
	/** @todo What if there is no item in the right hand - this would segfault then */
	} else if (hand->id == gi.csi->idLeft && RIGHT(ent)->item.t->holdTwoHanded) {
		/* Check for two-handed weapon */
		hand = INVDEF(gi.csi->idRight);
		weapon = CONTAINER(ent, hand->id)->item.t;
	} else
		return;

	assert(weapon);

	/* LordHavoc: Check if item is researched when in singleplayer? I don't think this is really a
	 * cheat issue as in singleplayer there is no way to inject fake client commands in the virtual
	 * network buffer, and in multiplayer everything is researched */

	for (containerID = 0; containerID < gi.csi->numIDs; containerID++) {
		if (INVDEF(containerID)->out < tu) {
			/* Once we've found at least one clip, there's no point
			 * searching other containers if it would take longer
			 * to retrieve the ammo from them than the one
			 * we've already found. */
			for (ic = ent->i.c[containerID]; ic; ic = ic->next)
				if (INVSH_LoadableInWeapon(ic->item.t, weapon)) {
					icFinal = ic;
					tu = INVDEF(containerID)->out;
					bestContainer = INVDEF(containerID);
					break;
				}
		}
	}

	/* send request */
	if (bestContainer)
		G_ActorInvMove(entnum, bestContainer, icFinal, hand, 0, 0, qtrue, quiet);
}
